from datetime import datetime
from typing import List, Optional
from dataclasses import dataclass

@dataclass
class UserProfile:
    id: str
    timestamp: int
    name: str
    is_legacy_profile: bool
    skin_variant: str
    cape_url: Optional[str] = None
    skin_url: Optional[str] = None

@dataclass
class Skin:
    id: str
    enabled: bool
    url: str
    variant: str
    alias: Optional[str] = None

@dataclass
class Cape:
    id: str
    enabled: bool
    url: str
    alias: str

@dataclass
class Profile:
    id: str
    name: str
    capes: List[Cape]
    skins: List[Skin]

@dataclass
class NameInformation:
    name_change_allowed: bool
    created_at: Optional[datetime] = None
    changed_at: Optional[datetime] = None


import ast
import json
import base64
import logging
from typing import Any, List, Dict, Optional

from async_mojang._types import UserProfile
from async_mojang._http_client import _HTTPClient
from async_mojang.errors import MojangError

_log = logging.getLogger(__name__)

_API_BASE_URL = "https://api.mojang.com"
_SESSIONSERVER_BASE_URL = "https://sessionserver.mojang.com"
_AUTHSERVER_BASE_URL = "https://authserver.mojang.com"

class API(_HTTPClient):
    async def get_uuid(self, username: str, timestamp: Optional[int] = None) -> Optional[str]:
        """Convert a Minecraft name to a UUID. Current implementation ignores the timestamp parameter."""
        url = f"{_API_BASE_URL}/users/profiles/minecraft/{username}"
        if timestamp:
            url += f"?at={timestamp}"

        try:
            resp = await self.request("GET", url, ignore_codes=[400])
            data = await resp.json()
            return data["id"]
        except (KeyError, json.decoder.JSONDecodeError):
            return None

    async def get_uuids(self, names: List[str]) -> Dict[str, str]:
        """Convert up to 10 usernames to UUIDs in a single network request."""
        if len(names) > 10:
            names = names[:10]

        resp = await self.request(
            "POST",
            f"{_API_BASE_URL}/profiles/minecraft",
            ignore_codes=[400],
            json=names,
        )
        data = await resp.json()
        if not isinstance(data, list):
            raise MojangError(response=resp)
        return {name_data["name"]: name_data["id"] for name_data in data}

    async def get_username(self, uuid: str) -> Optional[str]:
        """Convert a UUID to a username."""
        resp = await self.request(
            "GET",
            f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}",
            ignore_codes=[400],
        )

        try:
            data = await resp.json()
            return data["name"]
        except json.decoder.JSONDecodeError:
            return None

    async def get_profile(self, uuid: str) -> Optional[UserProfile]:
        """Get more information about a user from their UUID."""
        resp = await self.request(
            "GET",
            f"{_SESSIONSERVER_BASE_URL}/session/minecraft/profile/{uuid}",
            ignore_codes=[400],
        )

        try:
            properties = await resp.json()
            value = properties["properties"][0]["value"]
            data = ast.literal_eval(base64.b64decode(value).decode())

            return UserProfile(
                id=data["profileId"],
                timestamp=data["timestamp"],
                name=data["profileName"],
                is_legacy_profile=bool(data.get("legacy")),
                cape_url=data["textures"].get("CAPE", {}).get("url"),
                skin_url=data["textures"].get("SKIN", {}).get("url"),
                skin_variant=data["textures"].get("SKIN", {}).get("metadata", {}).get("model", "classic"),
            )
        except (KeyError, json.decoder.JSONDecodeError):
            return None

    async def get_blocked_servers(self) -> List[str]:
        """Get a list of SHA1 hashes of blacklisted Minecraft servers."""
        resp = await self.request("GET", f"{_SESSIONSERVER_BASE_URL}/blockedservers")
        data = await resp.text()
        return data.splitlines()


import asyncio
import logging
from typing import Any, List, Optional

import aiohttp

from http.client import HTTPConnection

from async_mojang.errors import (
    MojangError,
    BadRequest,
    Forbidden,
    NotFound,
    TooManyRequests,
    ServerError,
    Unauthorized,
)

_log = logging.getLogger(__name__)

class _HTTPClient:
    def __init__(
        self,
        session: Optional[aiohttp.ClientSession] = None,
        retry_on_ratelimit: Optional[bool] = False,
        ratelimit_sleep_time: Optional[int] = 60,
        debug_mode: Optional[bool] = False,
    ):
        self.ratelimit_sleep_time = ratelimit_sleep_time
        self.retry_on_ratelimit = retry_on_ratelimit
        self.session = session or aiohttp.ClientSession(
            headers={"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36"}
        )

        if debug_mode:
            HTTPConnection.debuglevel = 1
            logging.basicConfig()
            logging.getLogger().setLevel(logging.DEBUG)
            aiohttp_log = logging.getLogger("aiohttp.client")
            aiohttp_log.setLevel(logging.DEBUG)
            aiohttp_log.propagate = True

    async def request(
        self,
        method: str,
        url: str,
        ignore_codes: Optional[List[int]] = None,
        **kwargs: Any,
    ) -> Any:
        """Internal request handler using aiohttp"""
        async with self.session.request(method, url, **kwargs) as resp:
            _log.debug(f"Making API request: {method} {url}\n")
            await resp.read()  # Ensure the whole response is read

            if resp.ok:
                return resp

            if ignore_codes and resp.status in ignore_codes:
                return resp

            if resp.status == 400:
                raise BadRequest

            if resp.status == 401:
                raise Unauthorized

            if resp.status == 403:
                raise Forbidden

            if resp.status == 404:
                raise NotFound

            if resp.status == 429:
                if self.retry_on_ratelimit:
                    _log.warning(f"We are being ratelimited. Sleeping for {self.ratelimit_sleep_time} seconds.")
                    await asyncio.sleep(self.ratelimit_sleep_time)
                    return await self.request(method, url, ignore_codes, **kwargs)
                else:
                    raise TooManyRequests

            if resp.status >= 500:
                raise ServerError

            raise MojangError(response=resp)

    async def close(self):
        """Close the aiohttp session."""
        await self.session.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.session.close()


from async_mojang.api import API

from async_mojang.errors import (
    MojangError,
    LoginFailure,
    BadRequest,
    Forbidden,
    NotFound,
    TooManyRequests,
    ServerError,
    Unauthorized,
    MissingMinecraftLicense,
    MissingMinecraftProfile,
)


import aiohttp
from typing import Optional

class MojangError(Exception):
    """Base error class for all library-related exceptions in this file.
    Essentially, this could be caught to handle any exceptions thrown from this library.
    """

    def __init__(
        self,
        message: Optional[str] = None,
        response: Optional[aiohttp.ClientResponse] = None,
    ):
        if response:
            try:
                data = {}
                message = f"[HTTP {response.status}] - [{data.get('errorMessage', 'No error message provided')}]"
            except KeyError:
                message = f"[HTTP {response.status}] - {response.url}\n\n"
        else:
            if message:
                message = f"{self.__class__.__doc__}\n{message}"
            else:
                message = self.__class__.__doc__

        super().__init__(message)

class BadRequest(MojangError):
    """HTTP 400. The server could not process our request, likely due to an error of ours."""

class Unauthorized(MojangError):
    """HTTP 401. We are not authorized to access the requested resource.
    This can occur due to an invalid or expired Bearer token.
    """

class Forbidden(MojangError):
    """HTTP 403. We do not have permission to access the requested resource."""

class NotFound(MojangError):
    """HTTP 404. This resource does not exist."""

class TooManyRequests(MojangError):
    """HTTP 429. The server is ratelimiting us. Please wait for a bit before trying again."""

class ServerError(MojangError):
    """HTTP 5xx. The server encountered an unexpected condition that prevented it from fulfilling the request."""

class LoginFailure(MojangError):
    """The login process failed for some reason. This can occur due to an incorrect email or password."""

class MissingMinecraftLicense(MojangError):
    """The Microsoft account is valid, but it is missing a Minecraft license."""

class MissingMinecraftProfile(MojangError):
    """The account has a Minecraft license, but it hasn't created a profile yet."""


def _assert_valid_username(username: str) -> None:
    """Raises a ValueError if a username is considered invalid"""

    if len(username) < 3 or len(username) > 16:
        raise ValueError(
            "Invalid username. Username size must be between 3 and 16 characters"
        )

    if not username.isascii():
        raise ValueError("Invalid username. Username contains invalid characters")


from setuptools import setup, find_packages

setup(
    name="async-mojang",
    version="1.0.0",
    author="FroostySnoowman",
    description="An async Python wrapper for the Mojang API.",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/FroostySnoowman/Async-Mojang",
    packages=find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
    license="MIT",
    python_requires=">=3.7",
    install_requires=[
        "aiohttp",
    ],
    keywords=["mojang", "minecraft", "api", "mojang api", "minecraft api", "async mojang", "async minecraft"],
)